# BSD LICENSE
#
# Copyright (c) 2014, Ixia All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
# * Neither the name of the copyright holder nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#   Version 6.20
#   
#   File: addressTableUtils.tcl
#
#   Copyright   IXIA
#   All Rights Reserved.
#
#	Revision Log:
#	04/05/2000	DS
#
# Description: This file contains common procs used for ipAddressTable
#              manipulation
#
##################################################################################


########################################################################
# Procedure: ipAddressSetDefault
#
# This command sets the ipAddressTable & iAddressTableItem to defaults
#
#
########################################################################
proc ipAddressSetDefault {} \
{
    ipAddressTable setDefault
    ipAddressTableItem setDefault
}


########################################################################
# Procedure: updateIpAddressTable
#
# This command updates the ipAddressTable w/current ip & port objects
#
# Arguments(s):
#   chassis
#   card
#   port
#
########################################################################
proc updateIpAddressTable {chassis card port {write nowrite}} \
{
    set retCode 0

    if [ip get $chassis $card $port] {
        errorMsg "Error getting ip on port $chassis $card $port"
        set retCode 1
    }

    if [port get $chassis $card $port] {
        errorMsg "Error getting port $chassis $card $port"
        set retCode 1
    }

    ipAddressTableItem config -fromIpAddress            [ip cget -sourceIpAddr]
    ipAddressTableItem config -gatewayIpAddress         [ip cget -destDutIpAddr]
    ipAddressTableItem config -fromMacAddress           [port cget -MacAddress]
    ipAddressTableItem config -numAddresses             [port cget -numAddresses]

    if [ipAddressTableItem set] {
        errorMsg "Error setting ipAddressTableItem"
        set retCode 1
    }

    if [ipAddressTable addItem] {
        errorMsg "Error adding item to ipAddressTable"
        set retCode 1
    }

    ipAddressTable config -defaultGateway               [ipAddressTableItem cget -gatewayIpAddress]
    if [ipAddressTable set $chassis $card $port] {
        errorMsg "Error setting ipAddressTable on port $chassis $card $port"
        set retCode 1
    }

    if {$write == "write" && $retCode == 0} {
        if [protocolServer write $chassis $card $port] {
            errorMsg "Error writing protocol server on $chassis $card $port"
            set retCode 1
        }
    }

    return $retCode
}


