# BSD LICENSE
#
# Copyright (c) 2014, Ixia All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
# * Neither the name of the copyright holder nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#   Version 6.20
#   
#   File: defineTest.tcl
#   Copyright   IXIA
#   All Rights Reserved.
#
# Revision Log:
# 12-13-1999	DS	Genesis
#
# Description: This file contains the commands for all the tests. When a new test
# is added, create a new proc for that test here.
#
##################################################################################

namespace eval defineTest {} \
{
    variable defaultParms
    variable defaultParmType
    variable defaultCommands

#   moved parameter:numtrials to proc defineTest::registerCommand.
#   set defaultParms(numtrials)		        1
    set defaultParms(duration)		        20
    set defaultParms(waitResidual)		    2
    set defaultParms(testName) 		        ""
    set defaultParms(framesize)		        64
    set defaultParms(framesizeList)		    {}


#   set defaultParmType(numtrials)	        integer
    set defaultParmType(duration)	        integer
    set defaultParmType(waitResidual)	    integer
    set defaultParmType(testName) 	        string
    set defaultParmType(framesize) 	        integer
    set defaultParmType(framesizeList)      integerList

    set defaultCommands(config)             [array names defaultParms]
    set defaultCommands(cget)               [array names defaultParms]
    set defaultCommands(registerResultVars) {}
    set defaultCommands(start)              {}
}

########################################################################################
# Procedure: initializeDefineTest
#
# Description: This command is used to initialize the defineTest commands
#
########################################################################################
proc initializeDefineTest {} \
{
    defineTest::initialize
}

########################################################################################
# Procedure: defineTest::initialize
#
# Description: This command is used to initialize the defineTest commands
#
########################################################################################
proc defineTest::initialize {} \
{
    set retCode 0

    defineCommand registerCommand defineTest
    defineCommand registerMethod  defineTest registerCommand
    defineCommand registerMethod  defineTest registerParameter
    defineCommand registerMethod  defineTest registerTest

    return $retCode
}


########################################################################################
# Procedure: defineTest::registerCommand
#
# Description: This command registers a new Tcl test command
#
########################################################################################
proc defineTest::registerCommand {testCmd} \
{
    variable defaultParms
    variable defaultParmType
    variable defaultCommands

    set retCode 0

    defineCommand registerCommand $testCmd

    # assign global parms automatically on registration
    foreach parm [array names defaultParms] {
        defineCommand registerParameter -command $testCmd -parameter $parm -defaultValue $defaultParms($parm) -type $defaultParmType($parm)
    }

    defineTest registerParameter -command $testCmd -parameter numtrials      -defaultValue 1                 -type integer -validRange  {1 NULL}
    defineTest registerParameter -command $testCmd -parameter staggeredStart -defaultValue notStaggeredStart -type string  -validValues {staggeredStart notStaggeredStart true false}
    defineTest registerParameter -command $testCmd -parameter framesize      -defaultValue 64                -type integer 
    defineTest registerParameter -command $testCmd -parameter framesizeList  -defaultValue {}                -type integerList

    foreach method [array names defaultCommands] {
        defineCommand registerMethod $testCmd $method $defaultCommands($method)
    }

    return $retCode
}


########################################################################################
# Procedure: defineTest::registerParameter
#
# Description: This command registers a new parameter for this Tcl test command
#
########################################################################################
proc defineTest::registerParameter {args} \
{
    return [defineCommand registerParameter $args]
}


########################################################################################
# Procedure: defineTest::registerTest 
#
# Description: This command registers a new test (start command) for this Tcl command
#
########################################################################################
proc defineTest::registerTest  {testCmd testSubCmd {testSubCmdParameterList ""}} \
{
    return [defineCommand registerMethod $testCmd start $testSubCmd]
}



