# BSD LICENSE
#
# Copyright (c) 2014, Ixia All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
# * Neither the name of the copyright holder nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#   Version 6.20
#   
#   File: ftp.tcl
#   Copyright   IXIA
#   All Rights Reserved.
#
#	Description:  This file contains utilities FileTransfer
#
#	Revision Log:
#	Date		Author				Comments
#	-----------	-------------------	--------------------------------------------
#   10/25/2000  ds          		initial release
#
##################################################################################

package require FTP

########################################################################################
# Procedure:	putFtpLogWindow
#
# Description: 	Write a message to the ftp log window
#				
# Input:		textEntry
#
########################################################################################
proc putFtpLogWindow {textEntry} \
{
    logMsg $textEntry
}


########################################################################################
# Procedure:	ixFileSend
#
# Description: 	FTP protocol stuff... 
#
# Input:
#
########################################################################################
proc ixFileSend {ipAddress filename username password sourceDirectory destDirectory} \
{
    set retCode 0

    global ftpDone

    putFtpLogWindow "ipAddress=$ipAddress\nfilename=$filename\nusername=$username\npassword=$password\nsourceDirectory=$sourceDirectory\ndestDirectory=$destDirectory"

    set retCode 1
    if {[file isdirectory $destDirectory]} {
        if {![FTP::Open $ipAddress $username $password  -mode passive]} {
            putFtpLogWindow"Error opening FTP session"
            set retCode 2
        } else {
            if {![FTP::Get $sourceDirectory$filename $destDirectory$filename]} {
                putFtpLogWindow "Error getting file $sourceDirectory$filename"
                set retCode 3
            }
        }
        FTP::Close
    } else {
        putFtpLogWindow "Error - invalid destination directory $destDirectory"
        set retCode 4
    }

    set ftpDone $retCode

    if {$ftpDone == 1} {
        set retCode 0
    } else {
        set retCode 1
    }

    return $retCode
}


########################################################################################
# Procedure:	ixFileTransferStart
#
# Description: 	FTP protocol stuff... 
#
# Input:
#
########################################################################################
proc ixFileTransferStart { ipAddress filename username password sourceDirectory destDirectory } \
{
    global ixTclSvrHandle ftpDone

    set retCode 0

    ixFileSend $ipAddress $filename $username $password $sourceDirectory $destDirectory

    set ftpDone 0
    while {$ftpDone==0} {
        ;# Make a time out counter here
        ;# probably count to about 100 then
        ;# a failure should be returned
        set x 0
        after 1000 {set x 1}
        vwait x
    }
  
    switch $ftpDone {
        1 {
            ixPuts "File transfer complete."
        }
        2 {
            ixPuts "Error opening FTP session on $ipAddress, username:$username."
            set retCode 1
        }
        3 {
            ixPuts "Error transferring file $sourceDirectory$filename to $destDirectory$filename."
            set retCode 1
        }
        4 {
            ixPuts "Error - invalid destination directory $destDirectory"
            set retCode 1
        }
    }

    return $retCode
}


