# BSD LICENSE
#
# Copyright (c) 2014, Ixia All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
# * Neither the name of the copyright holder nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#   Version 6.20
#   
#   File: random.tcl
#
#   Copyright   IXIA
#   All Rights Reserved.
#
#	Revision Log:
#	02-03-2000	HSH
#
# Description: This file contains common procs used for generating random
#              numbers
#
##################################################################################


########################################################################
# Set the default randomSeed
# Now use a random number to be the randomSeed
########################################################################

global randomSeed

set randomSeed [expr int( [expr rand()] * 100 ) ]


########################################################################
# Procedure: RandomRange
# Should let range equal to "the total number of data" ( do not subtract 1 )!!
#
# range -  the range of random numbers 
#
########################################################################
proc RandomRange { range } \
{
 	
    return [ expr int( [ Random ] * $range ) ]
}

########################################################################
# Procedure: Random
# This gives a random decimal num.
# 
########################################################################

proc Random {} \
{
 	global randomSeed
 	set randomSeed [ expr ( $randomSeed * 9301 + 49297 ) %233280 ]
	return [ expr $randomSeed / double( 233280 ) ]
}

########################################################################
# Procedure: RandomInit
# 
# 
########################################################################
proc RandomInit {} \
{
    global randomSeed
    set randomSeed  [clock seconds]
}


########################################################################
# Procedure: RandomFromTo
# This gives a random num within the given range of numbers.
# 
########################################################################
proc RandomFromTo {from to} \
{
    global randomSeed

    set randomSeed [expr ($randomSeed*9301 + 49297) % 233280]
    set random     [expr $randomSeed/double(233280)]

    return [expr int($random * ($to - $from + 1)) + $from]
}

