# BSD LICENSE
#
# Copyright (c) 2014, Ixia All rights reserved.
# Copyright (c) 2014, Intel Corporation All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
# * Neither the name of the copyright holder nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#Config Ixia.
proc configIxia {chasId rxcard rxport rxda txcard txport txda} {
    set ::chasId   $chasId
    set ::rxcard   $rxcard
    set ::rxport   $rxport
    set ::txcard   $txcard
    set ::txport   $txport
    set ::rxPortList [list [list $chasId $rxcard $rxport]]
    set ::txPortList [list [list $chasId $txcard $txport]]
    set ::rxsa       [format "00 00 %02x 00 %02x 00" [expr $rxcard-1] [expr $rxport-1]]
    set ::rxda       [string trim $rxda "'"]
    set ::txsa       [format "00 00 %02x 00 %02x 00" [expr $txcard-1] [expr $txport-1]]
    set ::txda       [string trim $txda "'"]
    set ::rxGroup    $rxport 
    set ::txGroup    $txport 
}

#Config Port properties for DCB & PFC.
proc configPort {direction tc} {
    #direction :  rx tx
    #tc  :  4  8
    set ::direction     $direction
    set ::tc            $tc
    set ::[set direction]StreamNum   0
    if [array exists ::[set direction]StreamUPs] {
        unset ::[set direction]StreamUPs  
    }
    set da        [set ::[set direction]da]
    set card      [set ::[set direction]card]
    set port      [set ::[set direction]port]
    set portList  [set ::[set direction]PortList]

    port setFactoryDefaults $::chasId $card $port
    port config -speed                              10000
    port config -duplex                             full
    port config -flowControl                        true
    port config -directedAddress                    $da
    port config -autonegotiate                      false
    port config -transmitMode                       portTxModeAdvancedScheduler 
    port config -portMode                           port10GigLanMode
    port config -enableDataCenterMode               true
    port config -flowControlType                    ieee8021Qbb
    port config -pfcResponseDelayEnabled            0
    port config -pfcResponseDelayQuanta             255

    if [string equal "4" $tc] {
        port config -dataCenterMode                 fourPriorityTrafficMapping
        port config -pfcEnableValueListBitMatrix    "{1 1} {1 2} {1 4} {1 8}"
    } elseif [string equal "8" $tc] {
        port config -dataCenterMode                 eightPriorityTrafficMapping
        port config -pfcEnableValueListBitMatrix    "{1 1} {1 2} {1 4} {1 8} {1 16} {1 32} {1 64} {1 128}"
    } else {
        errorMsg "TCs can only be 4 or 8."
        return $::TCL_ERROR
    }
    if [port set $::chasId $card $port] {
        errorMsg "Could not call port set $::chasId $card $port"
        return $::TCL_ERROR
    }
    #Delete all streams from a port
    port reset $::chasId $card $port

    filter setDefault 
    if [filter set $::chasId $card $port] {
        errorMsg "Error calling filter set $::chasId $card $port"
        return $::TCL_ERROR
    }

    filterPallette setDefault 
    if [filterPallette set $::chasId $card $port] {
        errorMsg "Error calling filterPallette set $::chasId $card $port"
        return $::TCL_ERROR
    }

    capture setDefault
    capture config -fullAction                         wrap
    capture config -captureMode                        captureContinuousMode
    if [capture set $::chasId $card $port] {
        errorMsg "Error calling capture set $::chasId $card $port"
        return $::TCL_ERROR
    }

    streamRegion setDefault 
    streamRegion config -gapControlMode    streamGapControlFixed
    if [streamRegion set $::chasId $card $port] {
        errorMsg "Error calling streamRegion set $::chasId $card $port"
        return $::TCL_ERROR
    }

    #Write port properties to hardware
    puts "Write port $port properties to hardware."
    if [ixWritePortsToHardware portList] {
        errorMsg "Could not write port $port properties to hardware."
        return $::TCL_ERROR
    }
    after 500
}

#Configure all streams.
proc configStream {stream rate prio type {tc $::tc}} {
    #stream : UDP PFC
    #rate   : 1.0 100.0
    #prio   : 0 1 2 3 4 5 6 7 8
    #type   : on  off  - vt
    #       : XOFF XON - pfc
    #tc     : 8   4
    set ::stream  $stream

    stream setDefault 
    if {$rate == "0"} {                              
        stream config -enable                          false
    } else {
        stream config -enable                          true
    }
    stream config -percentPacketRate                   $rate
    stream config -rateMode                            streamRateModePercentRate
    stream config -asyncIntEnable                      true
    stream config -enableStatistic                     true
    stream config -enableSuspend                       false
    stream config -dma                                 contPacket
    stream config -priorityGroup                       "priorityGroup$prio"

    switch -exact -- $stream {
           "UDP" {
                   #Configure UDP     
                   stream config -sa                                  $::txsa
                   stream config -da                                  $::txda
                   stream config -framesize                           68
                   stream config -frameSizeMIN                        68
                   stream config -frameSizeMAX                        68
                   stream config -frameType                           "08 00"
     
                   protocol setDefault 
                   protocol config -name                              ipV4
                   protocol config -ethernetType                      ethernetII
                   protocol config -enable802dot1qTag                 vlanSingle
 
                   ip setDefault 
                   ip config -checksum                                "7a c0"
                   ip config -sourceIpAddr                            "0.0.0.0"
                   ip config -sourceIpAddrRepeatCount                 10
                   ip config -sourceClass                             classA
                   ip config -destIpAddr                              "0.0.0.0"
                   ip config -destIpAddrRepeatCount                   10
                   ip config -destClass                               classA
                   ip config -destMacAddr                             "00 DE BB 00 00 02"
                   ip config -destDutIpAddr                           "0.0.0.0"
                   if [ip set $::chasId $::txcard $::txport] {
                       errorMsg "Error calling ip set $::chasId $::txcard $::txport"
                       return $::TCL_ERROR
                   } 
 
                   udp setDefault 
                   udp config -sourcePort                            unassignedPort
                   udp config -destPort                              unassignedPort
                   udp config -checksum                              "b6 eb "
                   udp config -length                                26
                   if [udp set $::chasId $::txcard $::txport] {
                       errorMsg "Error calling udp set $::chasId $::txcard $::txport"
                       return $::TCL_ERROR
                   }
 
                   vlan setDefault
                   if [string equal "on" $type] {
                       vlan config -mode                            vIncrement
                       vlan config -repeat                          [expr 128/$::tc]
                   } elseif [string equal "off" $type] {
                       vlan config -vlanID                          [expr int([expr 32 * [expr rand()]])]
                   }
                   vlan config -userPriority                        $prio
                   if [vlan set $::chasId $::txcard $::txport] {
                       errorMsg "Could not call vlan set $::chasId $::txcard $::txport"
                       return $::TCL_ERROR
                   }
                   set bool 1
                   foreach item [array names ::txStreamUPs] { 
                       if {$prio == $::txStreamUPs($item)} {
                           incr bool -1
                           set id $item 
                           break 
                       }
                   }  
                   if { $bool } {
                       incr ::txStreamNum
                       set ::txStreamUPs($::txStreamNum) $prio
                       set id $::txStreamNum
                   }
                   if [stream set $::chasId $::txcard $::txport $id] {
                       errorMsg "Could not call stream set $::chasId $::txcard $::txport $id"
                       return $::TCL_ERROR
                   }
                   if [stream write $::chasId $::txcard $::txport $id] {
                       errorMsg "Could not write stream $::chasId $::txcard $::txport $id"
                   }
           }
           "PFC" {
                   #Configure PFC
                   stream config -sa                                 $::rxsa
                   stream config -da                                 $::rxda
                   stream config -framesize                          64
                   stream config -frameSizeMIN                       64
                   stream config -frameSizeMAX                       64
                   stream config -frameType                          "88 08"

                   protocol setDefault 
                   protocol config -name                             pauseControl
                   protocol config -ethernetType                     ethernetII

                   pauseControl setDefault 
                   pauseControl config -da                           $::rxda
                   pauseControl config -pauseTime                    255
                   pauseControl config -pauseControlType             ieee8021Qbb
                   pauseControl config -usePfcEnableValueList        0    

                   pauseControl config -priorityEnableVector         [format "%#04x" [expr 0x01<<$prio]]
                   for {set n 0; set paFrame ""; set pfcList ""} {$n < 8 } {incr n} { 
                       if {$type == "XOFF" && $prio == $n} {
                           append paFrame "FF FF "
                           append pfcList "{1 65535} "
                           continue
                       }
                       append paFrame "00 00 "
                       append pfcList "{0 0} "
                   }
                   pauseControl config -pauseFrame                  [string trimright $paFrame]
                   pauseControl config -pfcEnableValueList          [string trimright $pfcList]
                   if [pauseControl set $::chasId $::rxcard $::rxport] {
                       errorMsg "Error calling pauseControl set  $::chasId $::rxcard $::rxport"
                       set retCode $::TCL_ERROR
                   }
                   set bool 1
                   foreach item [array names ::rxStreamUPs] { 
                       if {$prio == $::rxStreamUPs($item)} {
                           incr bool -1
                           set id $item 
                           break 
                       }
                   }  
                  if { $bool } {
                       incr ::rxStreamNum
                       set ::rxStreamUPs($::rxStreamNum) $prio
                       set id $::rxStreamNum
                   }
                   if [stream set $::chasId $::rxcard $::rxport $id] {
                       errorMsg "Could not call stream set $::chasId $::rxcard $::rxport $id"
                       return $::TCL_ERROR
                   }
                   if [stream write $::chasId $::rxcard $::rxport $id] {
                       errorMsg "Could not write stream $::chasId $::rxcard $::rxport $id"
                       return $::TCL_ERROR
                   } 
          }
          default { 
                   errorMsg "Configure streams error."
                   return $::TCL_ERROR
          }
    }

    after 100
}

#Get the statistics counter for all stats 
proc getAllStats {direction {type None} {prio None}} {
    set card           [set ::[set direction]card]
    set port           [set ::[set direction]port]
    if [stat get statAllStats $::chasId $card $port] {
        puts "Could not read stats on port $port."
        return $::TCL_ERROR
    } 

    set framesSent [stat cget -framesSent]
    set framesRecv [stat cget -framesReceived]
    for {set upId 0; set recvSum 0} {$upId < $::tc} {incr upId} {
         set pfcUP$upId    [stat cget -rxPausePriorityGroup[set upId]Frames]
         incr recvSum [set pfcUP$upId]
    }

    if [stat getRate statAllStats $::chasId $card $port] {
        puts "Could not read stat rate on port $port."
        return $::TCL_ERROR
    }
    set framesSentRate [stat cget -framesSent]
    set framesRecvRate [stat cget -framesReceived]
    
    puts "+---------------------------------------------+"
    puts "[ format "| %-44s|" "Read Statistics On [string toupper $direction 0 0] Port $port Of Ixia."]"
    puts "+---------------------------------------------+"
    puts "[format "| %-12s| %-14s| %-14s|" "Type" "Count" "Unit"]"
    puts "+-------------+---------------+---------------+"
    switch -exact -- $type {
           "FramesSent" { 
                         puts "[format "| %-12s| %-14s| %-14s|" "Frames Sent" $framesSent "frames"]"
                         puts "+---------------------------------------------+"
                         after 50
                         return $framesSent
           }
           "RateSent" { 
                         puts "[format "| %-12s| %-14s| %-14s|" "Rate Sent" $framesSentRate "packets/sec"]"
                         puts "+---------------------------------------------+"
                         return $framesSentRate
           }
           "FramesRecv" { 
                         puts "[format "| %-12s| %-14s| %-14s|" "Frames Recv" $framesRecv "frames"]"
                         puts "+---------------------------------------------+"
                         after 50
                         return $framesRecv
           }
           "RateRecv" { 
                         puts "[format "| %-12s| %-14s| %-14s|" "Rate Recv" $framesRecvRate "packets/sec"]"
                         puts "+---------------------------------------------+"
                         after 50
                         return $framesRecvRate
           }
           "PfcsRecv" { 
                         puts "[format "| %-12s| %-14s| %-14s|" "Frames Sent" $framesSent "frames"]"
                         puts "+-------------+---------------+---------------+"
                         puts "[format "| %-12s| %-14s| %-14s|" "Frames Recv" $framesRecv "frames"]"
                         puts "+-------------+---------------+---------------+"
                         for {set upId 0} {$upId < $::tc} {incr upId} {
                             puts "[format "| %-12s| %-14s| %-14s|" "PFCs $upId Rcvd" [set pfcUP$upId] "frames"]"
                             puts "+-------------+---------------+---------------+"
                         }
                         puts "[format "| %-12s| %-14s| %-14s|" "Total  Recv" $recvSum "frames"]"
                         puts "+---------------------------------------------+"
                         after 50
                         if {$prio == "None"} {
                             return $recvSum
                         } else {
                             return [set pfcUP$prio]
                         }
           }
           default { 
                     puts "[format "| %-12s| %-14s| %-14s|" "Frames Sent" $framesSent "frames"]"
                     puts "+-------------+---------------+---------------+"
                     puts "[format "| %-12s| %-14s| %-14s|" "Actual Rate" $framesSentRate "packets/sec"]"
                     puts "+-------------+---------------+---------------+"
                     puts "[format "| %-12s| %-14s| %-14s|" "Frames Recv" $framesRecv "frames"]"
                     puts "+-------------+---------------+---------------+"
                     puts "[format "| %-12s| %-14s| %-14s|" "Actual Rate" $framesRecvRate "packets/sec"]"
                     puts "+-------------+---------------+---------------+"
                     for {set upId 0} {$upId < $::tc} {incr upId} {
                         puts "[format "| %-12s| %-14s| %-14s|" "PFCs $upId Rcvd" [set pfcUP$upId] "frames"]"
                         puts "+-------------+---------------+---------------+"
                     }
                     puts "[format "| %-12s| %-14s| %-14s|" "Total  Recv" $recvSum "frames"]"
                     puts "+---------------------------------------------+"
                     after 50
           }
    }
}
    
#Get per-stream transmit statistics on Tx port
proc getTxPerStreamStats {} {
    set nums [streamTransmitStats cget -numGroups]
    puts "+-------------------------------------------------------------+"
    puts "[format "| %-60s|" "Read $nums Streams On Tx Port $::txport Of Ixia."]"
    puts "+-------------------------------------------------------------+"
    puts "[format "| %-7s| %-3s| %-14s| %-14s| %-14s|" "Stream" "UP" "FramesSent" "Define Rate" "Actual Rate"]"
    #Get the statistics for each stream
    set rateSum 0
    set deRateSum 0
    set sentSum 0
    set aveRate    [streamTransmitStats cget -theoreticalAverageFrameRate]
    for {set streamId 1} {$streamId <= $nums} {incr streamId} {
        if [streamTransmitStats getGroup $streamId] {
           errorMsg "Could not get group $streamId on port $::txport."
           return $::TCL_ERROR
        }
        set frameRate  [streamTransmitStats cget -frameRate]
        set framesPSent [streamTransmitStats cget -framesSent]
        stream get  $::chasId $::txcard $::txport $streamId
        set deRate [expr int([stream cget -fpsRate])]
        
        puts "+--------+----+---------------+---------------+---------------+"
        puts "[format "| %-7s| %-3s| %-14s| %-14s| %-14s|" $streamId $::txStreamUPs($streamId) $framesPSent $deRate $frameRate]"
        incr rateSum $frameRate
        incr deRateSum $deRate
        incr sentSum $framesPSent
    }
    puts "+-------------------------------------------------------------+"
    puts "[format "| %-7s| %-3s| %-14s| %-14s| %-14s|" "Total" $nums $sentSum $deRateSum $rateSum]"
    puts "+-------------+---------------+---------------+---------------+"
    puts "[format "| %-12s| %-14s| %-14s| %-14s|" "Unit" "frames" "packets/sec" "packets/sec"]"
    puts "+-------------------------------------------------------------+"
    after 50
    }
    
#Get the captured frames on on Tx port
proc getTxCaptureFrames {type {prio None}} {
    puts "+--------------------------------------+"
    puts "[format "| %-37s|" "Analyse frames captured on port $::txport."]"
    if [capture get $::chasId $::txcard $::txport] {
        puts "Could not get capture frames on port $::txport"
        return $::TCL_ERROR
    } 
    set numFrames [capture cget -nPackets]
    puts "+--------------------------------------+"
    puts "[format "| %-16s| %-19s|" "Frames Captured" $numFrames]"
    if [expr $numFrames] {
        for {set frameId 1} {$frameId <= $numFrames} {incr frameId} {
            captureBuffer get $::chasId $::txcard $::txport $frameId $frameId
            captureBuffer getframe 1
            set capFrame     [captureBuffer cget -frame]
            set etherType    [lrange $capFrame 12 13]
      
            switch -exact -- $etherType {
                   "08 00" { 
                             set types "IPv4 Frame" 
                             if {$type == "UDP"} {
                                 return 1
                             }
                   }
                   "86 DD" { 
                             set types "IPv6 Frame"
                             if {$type == "UDP"} {
                                 return 1
                             }
                   }
                   "81 00" { 
                             set types "VLAN-tagged Frame"
                             if {$type == "UDP"} {
                                 return 1
                             }
                   }
                   "88 08" {  
                             set types "MAC Control Frame"
                             set ctrlOpcode   [lrange $capFrame 14 15]
                             switch -exact -- $ctrlOpcode {
                                    "00 01" { 
                                              set fc    "IEEE 802.3x PAUSE Frame"
                                              if {$type == "FC-PAUSE"} {
                                                  return 1
                                              } 
                                    }
                                    "01 01" { 
                                              set prioEnVector    [lrange $capFrame 16 17]
                                              if {[lrange $capFrame 17 17] == [format "%02x" [expr 0x01<<$prio]]} {
                                                  for {set timeId 0; set start 17; set ret ""} {$timeId < $::tc} {incr timeId} {
                                                       set Time$timeId  [lrange $capFrame [incr start] [incr start]]
                                                       if {[set Time$timeId] == "00 00" && $ret != "XOFF"} {
                                                           set ret "XON"
                                                       } else {
                                                           set ret "XOFF"
                                                       }  
                                                  }
                                                  if {$type == $ret} {                          
                                                      puts "+-----------------+--------------------+"
                                                      puts "[format "| %-16s| %-19s|" "Frames Analysed" $frameId]"
                                                      puts "+-----------------+--------------------+"
                                                      puts "[format "| %-16s| %-19s|" "Frames Type" $types]"
                                                      puts "+-----------------+--------------------+"
                                                      puts "[format "| %-16s| %-19s|" " " "PFC $type Frame"]"
                                                      puts "+-----------------+--------------------+"
                                                      puts "[format "| %-16s| %-19s|" "Control Opcode" $ctrlOpcode]"
                                                      puts "+-----------------+--------------------+"
                                                      puts "[format "| %-16s| %-19s|" "Priority Vector" $prioEnVector]"
                                                      puts "+-----------------+--------------------+"
                                                      puts "[format "| %-16s| %-19s|" " " $prio]"
                                                      for {set timeId 0; set start 17; set ret ""} {$timeId < $::tc} {incr timeId} {
                                                          puts "+-----------------+--------------------+"
                                                          puts "[format "| %-16s| %-19s|" "Time$timeId" [set Time$timeId]]"
                                                      }
                                                      puts "+--------------------------------------+"
                                                      after 50
                                                      return 1
                                                  }
                                              }    
                                    } 
                             }               
                   }
                   default { puts "Unknown Frame" }
            }
        }
    } 
    puts "+--------------------------------------+"
    return 0
}
        
#Clear all statistic counters on Tx/Rx port
proc clearStats {direction} {
    set portList  [set ::[set direction]PortList] 
    if [ixClearStats portList] {
        return $::TCL_ERROR
    }
    after 100
}

#Start transmission on Tx/Rx port
proc startTransmit {direction} {
    set port      [set ::[set direction]port]
    set portList  [set ::[set direction]PortList] 
    puts "+------------------------------------+"
    puts "[format "| %-35s|" "Start transmit on $direction port $port."]"
    if [ixStartTransmit portList] {
        return $::TCL_ERROR
    }
    puts "+------------------------------------+"
    after 500
}

#Stop transmission on Tx/Rx port
proc stopTransmit {direction} {
    set port      [set ::[set direction]port]
    set portList  [set ::[set direction]PortList] 
    puts "+------------------------------------+"
    puts "[format "| %-35s|" "Stop transmit on $direction port $port."]"
    if [ixStopTransmit portList] {
        return $::TCL_ERROR
    }
    puts "+------------------------------------+"
    after 500
}

#Start capture on Tx/Rx port
proc startCapture {direction} {
    set card      [set ::[set direction]card]
    set port      [set ::[set direction]port]
    puts "+------------------------------------+"
    puts "[format "| %-35s|" "Start capture on $direction port $port."]"
    if [ixStartPortCapture $::chasId $card $port] {
        return $::TCL_ERROR
    }
    puts "+------------------------------------+"
}

#Stop capture on Tx/Rx port
proc stopCapture {direction} {
    set card      [set ::[set direction]card]
    set port      [set ::[set direction]port]
    puts "+------------------------------------+"
    puts "[format "| %-35s|" "Stop capture on $direction port $port."]"
    if [ixStopPortCapture $::chasId $card $port] {
       return $::TCL_ERROR
    }
    puts "+------------------------------------+"
}

#Get Per-stream transmit statistics on Tx port
proc perStreamTxStats {} {
    puts "+------------------------------------+"
    puts "[format "| %-35s|" "Get Per-stream stats on Tx port $::txport."]"
    if [streamTransmitStats get $::chasId $::txcard $::txport 1 $::txStreamNum] {
       return $::TCL_ERROR
    }
    puts "+------------------------------------+"
    after 1000
}

#Clean, Logoff and Disconnect
proc clean {server portList}  {
    #Let go of the ports that we reserved
    ixClearOwnership $portList
    #Log off user
    ixLogout
    #End a test and cleanup all variables
    cleanUp
}
