# BSD LICENSE
#
# Copyright (c) 2014, Ixia All rights reserved.
# Copyright (c) 2014, Intel Corporation All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
# * Neither the name of the copyright holder nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

proc ping6 {destIP destMAC chasId card port } {

	set retCode $::TCL_OK 
	set streamId 1
	set portList [list [list $chasId $card $port]]
	set portIP "fe80:0000:0000:0000:021e:67ff:fe0d:b60a"

	# Set ports to factory defaults. Dumps out on error. 
	port setFactoryDefaults $chasId $card $port
	port setDefault

	# Writes port properties in hardware 
	if {[ixWritePortsToHardware portList]} { 
		return $::TCL_ERROR 
	} 
 
# Check the link state of the ports 
	if {[ixCheckLinkState portList]} { 
		return $::TCL_ERROR 
	}

	protocol setDefault
	protocol config -ethernetType ethernetII 
	protocol config -name ipV6 

	icmpV6 setDefault
	icmpV6 setType icmpV6EchoRequestMessage 

	icmpV6Informational setDefault
	icmpV6Informational config -identifier 58275
	icmpV6Informational config -sequenceNumber 1

	if {[icmpV6 set $chasId $card $port]} { 
		return $::TCL_ERROR 
	}

	ipV6 setDefault
	ipV6 config -sourceAddr $portIP
	ipV6 config -sourceMask 64
	ipV6 config -destAddr $destIP
	ipV6 config -destMask 64
	ipV6 config -nextHeader ipV4ProtocolIpv6Icmp
	ipV6 config -trafficClass 3
	ipV6 clearAllExtensionHeaders
 
	if {[ipV6 addExtensionHeader ipV4ProtocolIpv6Icmp]} {
		return $::TCL_ERROR 
	}

	if {[ipV6 set $chasId $card $port]} { 
		return $::TCL_ERROR 
	}

	stream config -numFrames 1
	stream config -dma stopStream 
	stream config -framesize 122
	stream config -rateMode usePercentRate 
	stream config -percentPacketRate 1 
	stream config -sa "00 00 01 00 02 00"
	stream config -da $destMAC

	if {[stream set $chasId $card $port $streamId]} { 
		return $::TCL_ERROR 
	}

	if [ixWriteConfigToHardware portList] { 
		return -code error 
	}	

	if [ixClearStats portList] { 
		return -code error 
	}

	after 1000

	ixStartCapture portList
	ixStartTransmit portList

	after 1000

	if {[ixCheckTransmitDone portList] == $::TCL_ERROR} {
		return -code error
	} else {
		ixPuts "Transmission is complete"
	}	

	if [ixStopCapture portList] {
		return -code error
	}

	captureBuffer get $chasId $card $port
	if {[captureBuffer cget -numFrames] == 0} {
		ixPuts "No packets received"
	} else {
		captureBuffer getframe 1
		set data [captureBuffer cget -frame]
		icmpV6 decode $data $chasId $card $port
		ipV6   decode $data $chasId $card $port
		if {[icmpV6 cget -type] == 135 && [ipV6 cget -sourceAddr] == $destIP} {
			ixPuts "64 bytes from"
		}
	}
#	ixClearOwnership $portList
#	ixDisconnectFromChassis $hostname
#	if [isUNIX] {
#		ixDisconnectTclServer $hostname
#	}
#    cleanUp
}
